@@ -1,7 +1,7 @@
 /***********************************************************
 * Rocks'n'Diamonds -- McDuffin Strikes Back!               *
 *----------------------------------------------------------*
-* (c) 1995-2002 Artsoft Entertainment                      *
+* (c) 1995-2006 Artsoft Entertainment                      *
 *               Holger Schemel                             *
 *               Detmolder Strasse 189                      *
 *               33604 Bielefeld                            *
@@ -16,14 +16,6 @@
 
 #include "main.h"
 
-/* number of element button columns and rows in the edit window */
-#define MAX_ELEM_X		4
-#define MAX_ELEM_Y		10
-
-extern int element_shift;
-extern int editor_element[];
-extern int elements_in_list;
-
 void CreateLevelEditorGadgets();
 void FreeLevelEditorGadgets();
 void UnmapLevelEditorGadgets();
@@ -32,5 +24,9 @@ void HandleLevelEditorKeyInput(Key);
 void HandleLevelEditorIdle();
 void HandleEditorGadgetInfoText(void *ptr);
 void RequestExitLevelEditor(boolean);
+void PrintEditorElementList();
+
+void DumpBrush();
+void DumpBrush_Small();
 
 #endif
