@@ -93,7 +93,7 @@ void BlitScreenToBitmap_EM(Bitmap *target_bitmap)
   }
 }
 
-void blitscreen(void)
+void BackToFront_EM(void)
 {
   static boolean scrolling_last = FALSE;
   int left = screen_x / TILEX;
@@ -101,6 +101,8 @@ void blitscreen(void)
   boolean scrolling = (screen_x % TILEX != 0 || screen_y % TILEY != 0);
   int x, y;
 
+  SyncDisplay();
+
   if (redraw_tiles > REDRAWTILES_THRESHOLD || scrolling || scrolling_last)
   {
     /* blit all (up to four) parts of the scroll buffer to the backbuffer */
@@ -126,6 +128,8 @@ void blitscreen(void)
     }
   }
 
+  FlushDisplay();
+
   for (x = 0; x < MAX_BUF_XSIZE; x++)
     for (y = 0; y < MAX_BUF_YSIZE; y++)
       redraw[x][y] = FALSE;
@@ -134,6 +138,11 @@ void blitscreen(void)
   scrolling_last = scrolling;
 }
 
+void blitscreen(void)
+{
+  BackToFront_EM();
+}
+
 static void DrawLevelField_EM(int x, int y, int sx, int sy,
 			      boolean draw_masked)
 {
@@ -869,14 +878,16 @@ void RedrawPlayfield_EM(boolean force_redraw)
 	blitplayer(&ply[i]);
 
       blitscreen();
-      FlushDisplay();
+
       Delay(wait_delay_value);
 
       /* scroll second step to align at full tile size */
       screen_x -= dxx;
       screen_y -= dyy;
 
+#if 0
       SyncDisplay();
+#endif
 
       animscreen();
 
@@ -884,7 +895,7 @@ void RedrawPlayfield_EM(boolean force_redraw)
 	blitplayer(&ply[i]);
 
       blitscreen();
-      FlushDisplay();
+
       Delay(wait_delay_value);
     }
 
@@ -912,7 +923,36 @@ void RedrawPlayfield_EM(boolean force_redraw)
 			    sy - offset_y > screen_y ? sy - offset_y :
 			    screen_y);
 
+#if 0
+  printf("::: (%d, %d) => (%d, %d) [(%d, %d), (%d, %d)] [%d, %d] [%d / %d]\n",
+	 screen_x_old, screen_y_old,
+	 screen_x, screen_y,
+	 ply[max_center_distance_player_nr].oldx,
+	 ply[max_center_distance_player_nr].x,
+	 ply[max_center_distance_player_nr].oldy,
+	 ply[max_center_distance_player_nr].y,
+	 sx, sy,
+	 ABS(screen_x - screen_x_old),
+	 ABS(screen_y - screen_y_old));
+#endif
+
 #if 1
+
+#if 1
+  /* prevent scrolling further than double player step size when scrolling */
+  if (ABS(screen_x - screen_x_old) > 2 * stepsize)
+  {
+    int dx = SIGN(screen_x - screen_x_old);
+
+    screen_x = screen_x_old + dx * 2 * stepsize;
+  }
+  if (ABS(screen_y - screen_y_old) > 2 * stepsize)
+  {
+    int dy = SIGN(screen_y - screen_y_old);
+
+    screen_y = screen_y_old + dy * 2 * stepsize;
+  }
+#else
   /* prevent scrolling further than double player step size when scrolling */
   if (ABS(screen_x - screen_x_old) > 2 * stepsize ||
       ABS(screen_y - screen_y_old) > 2 * stepsize)
@@ -923,6 +963,8 @@ void RedrawPlayfield_EM(boolean force_redraw)
     screen_x = screen_x_old + dx * 2 * stepsize;
     screen_y = screen_y_old + dy * 2 * stepsize;
   }
+#endif
+
 #else
   /* prevent scrolling further than player step size when scrolling */
   if (ABS(screen_x - screen_x_old) > stepsize ||
@@ -997,11 +1039,13 @@ void RedrawPlayfield_EM(boolean force_redraw)
   for (i = 0; i < MAX_PLAYERS; i++)
     blitplayer(&ply[i]);
 
+#if 0
+#if 0
   SyncDisplay();
+#endif
 
   blitscreen();
-
-  FlushDisplay();
+#endif
 }
 
 void game_animscreen(void)
